/*
 *  auditd.h
 *  Linux Basic Security Module 
 *  Copyright (c) 2000 The Regents of the University of California
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *
 *  3. The name of the author may not be used to endorse or promote products
 *  derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND 
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 *  SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 *  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef AUDITD_H
#define AUDITD_H

#define BSM_MAJOR_VERSION 0 
#define BSM_MINOR_VERSION 6
#define BSM_PATCHLEVEL 0

#define PATHLEN 256	/* 255 chars in path plus \0 */

/* ioctl modes */
#define BSM_START		0  /* start auditing */
#define BSM_STOP		1  /* stop auditing */
#define BSM_DEFAULT		2  /* Apply default config */
#define BSM_CONFIG_BUF	3  /* Set memory buffer size */
#define BSM_CONFIG_PID	4  /* Audit one process */
#define BSM_CONFIG_GID	5  /* One group */
#define BSM_CONFIG_UID	6  /* One user */
#define BSM_BUFSIZE		7  /* returns the current buffer size */
#define BSM_WRITTEN		8  /* Daemon wrote out some data */
#define BSM_LOST		9  /* returns # of lost events */

/* the default memory buffer size */
#define BSM_DEFAULT_BUF 500000

/* Options structure */
typedef struct _pref
{
	int default_flag; /* using the defaults */
	int auditing_pid; /* audit one pid, gid, uid in particular */
	int auditing_gid;
	int auditing_uid;
	int pid; /* pid being audited */	 
	int gid;	
	int uid;
	int custom_bufsize; /* not using the default mem buffer size */
	int bufsize;
} pref;

pref* new_prefs(void);
void set_prefs(int , char**, pref* );
void set_filename(int);
void bad_prefs(char);
void apply_prefs(void);
void sig_handler(int);
int main(int, char**);
void error (char *);
void error_sys(char *);



#endif  /* AUDITD_H */
