/*
 *  praudit.c
 *  Linux Basic Security Module 
 *  Copyright (c) 2000 The Regents of the University of California
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *
 *  3. The name of the author may not be used to endorse or promote products
 *  derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND 
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 *  SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 *  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <errno.h>
#include <fstream.h>
#include <iostream.h>
#include <stdio.h>		// for perror()
#include <time.h>		// ctime()


#include "praudit.h"

ifstream auditTrail;
audit_header header; // easier to make record headers global
int bytes_read = 0;

/* spit an error and bail */
void Error (char *s)
{
	cerr << s << endl;
	exit(1);
}
/* uses perror to get the errno info */
void ErrorSys(char *s)
{
	perror(s);
	exit(1);	
}

// these fucntions all format and output events that belongs to their class

// should only get this class at the end of the trail
void OutputNo()
{
	if (header.event_size != 0) // then something may be corrupt
		cout << "Read invalid class before EOF, quitting" << endl;
	
	cout << endl << "finished, processed " << bytes_read << " bytes." << endl;
	return;	
}

void OutputFr()
{
	cout << "not here yet" << endl;
	return;
}

// also EX, since that just contains execve() and looks similar
void OutputPc()
{
	pc_event theEvent;
	int test = 0, tempTimeLen;
	char* tempTime;
	
	// read in the event and test
	auditTrail.read( (char*) &theEvent, sizeof (theEvent) );
		test = auditTrail.gcount();   // returns the # of bytes read
		bytes_read += test;
		if (test < sizeof(theEvent))
			Error("Insufficient record size for this class (audit trail\
					corrupted?); exiting\n");

	
	cout << "read " << test << " bytes" << endl;


	// convert seconds in timeval (should decompose to a time_t and work)
	tempTime = ctime(&header.time.tv_sec);
	
	// munge the newline in tempTime to a space for pretty printing
	tempTimeLen = strlen(tempTime);
	tempTime[tempTimeLen-1] = 32; 
	
	switch(header.event_id)
	{
		case BSM_CHDIR:
			cout << "header," << header.event_size << ","  << "chdir()," <<
					tempTime << "+" << header.time.tv_usec << endl;;
			cout << "path," << theEvent.path << "," << endl;
			cout << "subject," << theEvent.euser_id << "," << theEvent.egroup_id << ","
				<< theEvent.user_id << "," << theEvent.group_id << ","
				<< theEvent.process_id
				<< endl; 
			cout << "return," << theEvent.return_val << endl;
			break;
		case BSM_CHROOT:
			cout << "header," << header.event_size << ","  << "chroot()," <<
					tempTime << "+" << header.time.tv_usec << endl;
			cout << "path," << theEvent.path << "," << endl;
			cout << "subject," << theEvent.euser_id << "," << theEvent.egroup_id << ","
				<< theEvent.user_id << "," << theEvent.group_id << ","
				<< theEvent.process_id
				<< endl; 
			cout << "return," << theEvent.return_val << endl;
			break;
		case BSM_EXECVE:
			cout << "header," << header.event_size << ","  << "execve()," <<
					tempTime << "+" << header.time.tv_usec << endl;
			cout << "path," << theEvent.path << "," << endl;
			cout << "subject," << theEvent.euser_id << "," << theEvent.egroup_id << ","
				<< theEvent.user_id << "," << theEvent.group_id << ","
				<< theEvent.process_id
				<< endl; 
			cout << "return," << theEvent.return_val << endl;
			break;
		default:
			cout << "Warning: Invalid event ID, got " << header.event_id << endl;
	}

	return;
}


// Skips the extra (empty) bytes at the end of a buffer
void HandleEndBuf()
{

	int test = 0;
	char* skipBytes;
	
	auditTrail.read( skipBytes, header.event_size);
		test = auditTrail.gcount();   // returns the # of bytes read
		bytes_read += test;
	
	cout << "end of buffer, skipped " << test << " bytes" << endl;	
	
	return;
}

int main(int argc, char* argv[])
{	
	int test = 0, done = 0;
	time_t test2;

	
	// open the file and read in the first header
		
	auditTrail.open(argv[1] );
	if ( !( auditTrail.is_open() ))
		ErrorSys("Couldn't open the file");


	while (!done)
	{

		
		// read in a header and test it
		auditTrail.read( (char*) &header, sizeof(header) );
		test = auditTrail.gcount();   // returns the # of bytes read
		bytes_read += test;
		if (test < sizeof(header))
			Error("Reached an unexpected EOF (audit trail corrupted?); exiting\n");
		
		
		cout << "read " << test << " bytes" << endl;
		cout << "header.event_class=" << header.event_class << " ";
		cout << "header.event_id=" << header.event_id << " ";
		cout << "header.event_size=" << header.event_size << endl;

		
		switch(header.event_class)
		{
			case BSM_CLASS_NO:
				OutputNo();
				done = 1;
				break;
			case BSM_CLASS_FR:
				OutputFr();
				break;
			case BSM_CLASS_EX:
			case BSM_CLASS_PC:
				OutputPc();
				break;
			case BSM_CLASS_ENDBUF:
				HandleEndBuf();
				break;
				
			default:
				cout << "Warning: Invalid audit class, got " << header.event_class << endl;
		}
		
		cout << endl;

	}
	
	
	auditTrail.close();
	exit(0);
}
