/*
 *  praudit.h
 *  Linux Basic Security Module 
 *  Copyright (c) 2000 The Regents of the University of California
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *
 *  3. The name of the author may not be used to endorse or promote products
 *  derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND 
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 *  SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 *  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#ifndef PRAUDIT_H
#define PRAUDIT_H

#define BSM_MAJOR_VERSION 0 
#define BSM_MINOR_VERSION 6
#define BSM_PATCHLEVEL 0

// duplicating definitions so we don't have to include the kernel audit.h --
// causes type incompatibilities and such

#include <sys/time.h>	// timeval - compatible with kernel's def, both break down to
						// a pair of longs


/* definition of the audit class bitmasks - configuration */
#define BSM_CLASS_NO 0 //0x00000000 /* invalid class */
#define BSM_CLASS_FR 1 //0x00000001 /* file read */
#define BSM_CLASS_FW 2 //0x00000002 /* file write */
#define BSM_CLASS_FA 3 //0x00000004 /* access file attributes */ 
#define BSM_CLASS_FM 4 //0x00000008 /* modify file attributes */
#define BSM_CLASS_FC 5 //0x00000010 /* create file */
#define BSM_CLASS_FD 6 //0x00000020 /* delete file */
#define BSM_CLASS_CL 7 //0x00000040 /* close file */
#define BSM_CLASS_PC 8 //0x00000080 /* process */
#define BSM_CLASS_NT 9 //0x00000100 /* network */
#define BSM_CLASS_IP 10 //0x00000200 /* IPC */
#define BSM_CLASS_NA 11 //0x00000400 /* nonattributable */
#define BSM_CLASS_AD 12 //0x00000800 /* admin */
#define BSM_CLASS_LO 13 //0x00001000 /* login-logout */
#define BSM_CLASS_AP 14 //0x00002000 /* application-caused events */
#define BSM_CLASS_IO 15 //0x00004000 /* ioctl */
#define BSM_CLASS_EX 16 //0x00008000 /* exec */
#define BSM_CLASS_OT 17 //0x00010000 /* other */
#define BSM_CLASS_ALL 18 //0x00020000 /* all events */
#define BSM_CLASS_ENDBUF 50	/* end of current buffer */


/* defines for the audit events themselves */
#define BSM_NULL	 0
#define BSM_EXIT	 1
#define BSM_FORK	 2
#define BSM_OPEN	 3
#define BSM_CREAT	 4
#define BSM_LINK	 5
#define BSM_UNLINK	 6

#define BSM_MKNOD	 9
#define BSM_CHMOD	10
#define BSM_CHOWN	11
#define BSM_UMOUNT	12
#define BSM_JUNK	13
#define BSM_ACCESS	14
#define BSM_KILL	15
/* more later */

/* ex class */
/* #define BSM_EXEC	 7	no exec() call in linux */
#define BSM_EXECVE	23	/* execve() */

/* pc class */
/* BSM_EXEC			 7*/
#define BSM_CHDIR	 8	/* chdir() */
/* BSM_EXECVE		23*/
#define BSM_CHROOT	24	/* chroot() */



/* Struct definitions for the audit events */
#define PATHLEN 256	/* 255 chars in path plus \0 */

/* token header, also used to mark the end of a buffer/trail
   End of buffer headers will have class ENDBUF and the event size will
   be the number of empty bytes to skip to get to the beginning of the next
   buffer. End of trail headers will have class 0 and size 0 */
typedef struct
{
	unsigned int event_class;	/* class of the event */
	unsigned int event_id;		/* number of the event */
	unsigned int event_size;	/* size of the event struct - don't include header */
								/* since it's always the same 20 bytes */
	struct timeval time;		/* this is obvious...... */
} audit_header;

/* file reads */
typedef struct
{
  int process_id;
  int user_id;
  int euser_id;
  int group_id;
  int egroup_id;
  int path_len;
  char path[PATHLEN];
} fr_event;

/* file writes */
typedef struct
{
  int process_id;
  int user_id;
  int euser_id;
  int group_id;
  int egroup_id;
  int session_id;
  int device_id;
  int path_len;
  char path[PATHLEN];
} fw_event;

/* file attribute access */
/* file attribute mod */
/* file creation */
/* file deletion */
/* file close */

/* process and exec calls share one struct */
typedef struct
{
	/* subject token */
	int process_id;
	int user_id;
	int euser_id;
	int group_id;
	int egroup_id;
	/* path token */
	char path[PATHLEN];
	int path_len;
	/* return token */
	/*int error_status;*/
	int return_val;
} pc_event;

/* network event */
typedef struct
{
  int process_id;
  int user_id;
  int euser_id;
  int group_id;
  int egroup_id;
  int socket_type;
  int local_port;
  int local_address;
  int remote_port;
  int remote_address;
  int len;
  char path[PATHLEN];
} nt_event;

/* ipc event */
typedef struct
{
  int process_id;
  int user_id;
  int euser_id;
  int group_id;
  int egroup_id;
  int path_len;
  char path[PATHLEN];
} ip_event;

/* admin event */

/* login/logout */
typedef struct
{
  int process_id;
  int user_id;
  int euser_id;
  int group_id;
  int egroup_id;
  int path_len;
  char path[PATHLEN];
} lo_event;

/* ioctl event */
typedef struct
{
  int process_id;
  int user_id;
  int euser_id;
  int group_id;
  int egroup_id;
  int path_len;
  char path[PATHLEN];
} ic_event;

/* misc event */

int main(int, char**);
void error (char *);
void error2(char *);
void OutputFr(void);
void OutputPc(void);
void HandleEndBuf(void);

#endif  /* PRAUDIT_H */
